
From: Chris Wright <chrisw@osdl.org>

The topdown changes in 2.6.12-rc1 can cause large allocations with large
stack limit to fail, despite there being space available.  The
mmap_base-len is only valid when len >= mmap_base.  However, nothing in
topdown allocator checks this.  It's only (now) caught at higher level,
which will cause allocation to simply fail.  The following change restores
the fallback to bottom-up path, which will allow large allocations with
large stack limit to potentially still succeed.

Signed-off-by: Chris Wright <chrisw@osdl.org>
Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 mm/mmap.c |    4 ++++
 1 files changed, 4 insertions(+)

diff -puN mm/mmap.c~mmap-topdown-fix-for-large-stack-limit-large-allocation mm/mmap.c
--- 25/mm/mmap.c~mmap-topdown-fix-for-large-stack-limit-large-allocation	2005-05-31 02:43:55.000000000 -0700
+++ 25-akpm/mm/mmap.c	2005-05-31 02:43:55.000000000 -0700
@@ -1267,6 +1267,9 @@ arch_get_unmapped_area_topdown(struct fi
 			return (mm->free_area_cache = addr-len);
 	}
 
+	if (mm->mmap_base < len)
+		goto bottomup;
+
 	addr = mm->mmap_base-len;
 
 	do {
@@ -1288,6 +1291,7 @@ arch_get_unmapped_area_topdown(struct fi
 		addr = vma->vm_start-len;
 	} while (len < vma->vm_start);
 
+bottomup:
 	/*
 	 * A failed mmap() very likely causes application failure,
 	 * so fall back to the bottom-up function here. This scenario
_
